--====================================================================
-- command processor
--====================================================================
-- called as early as possible in per-frame processing
-- otherwise there are brief visual glitches as one thing changes
-- but others don't

-- previous selection: restore immediately after recognizing command

selP = nil
sameSel = false

-- table of commands:
-- command text (name of selected object) = command function:

cmds = {
 }

carpet_cmd = function (this_date)

  local sel = celestia:getselection()
  if (sel == selP) then sameSel = true; return 0 end
  sameSel = false

  local sname = sel:name()

    todo = cmds[sname]
    if todo then 
	todo(this_date)
    	if (selP ~= nil) then celestia:select(selP) end -- restore previous selection

--  	local s = "Hale command:  name= "..sname
-- 	celestia:flash(s)
	return 1
    end

-- This selection is not recognized as a command: 
-- save it for reversion after next command

	selP = sel
	return 0
end

------------- scripted orbit which invokes the command processor

Carpet_Command_Processor = function (sscvals)

	local orbit = {}
	orbit.boundingRadius = 1e5

	
	function orbit:position (tjd)
		carpet_cmd(tjd)
		return 0, 0, 0
	end

return orbit
end
------------------------------------------------------
-- ------------------------
-- generic motion functions
-- ------------------------

InitMoveTable = function (obj,first,last)
	obj.T0 = 1e18
	obj.T1 = 0

	obj.P0 = first
	obj.P1 = last

	obj.Pos0 = first
	obj.Pos1 = first

	obj.TPrev = 1e18
	obj.PosPrev = first

	obj.V = 0
end


MoveToP1 = function (obj,mydate)
	obj.Pos0 = obj.PosPrev
	obj.Pos1 = obj.P1
	obj.V = (obj.P1-obj.P0)/dT
	obj.T0 = obj.TPrev
	local sf = math.abs((obj.P1-obj.PosPrev)/(obj.P1-obj.P0))
	obj.T1 = obj.TPrev+dT*sf
end

MoveToP0 = function (obj,mydate)
	obj.Pos0 = obj.PosPrev
	obj.Pos1 = obj.P0
	obj.V = (obj.P0-obj.P1)/dT
	obj.T0 = obj.TPrev
	local sf = math.abs((obj.P0-obj.PosPrev)/(obj.P1-obj.P0))
	obj.T1 = obj.TPrev+dT*sf
end

MoveStop = function (obj,mydate)
	obj.Pos0 = obj.PosPrev
	obj.Pos1 = obj.PosPrev
	obj.V = 0
	obj.T0 = mydate
	obj.T1 = mydate
	obj.TPrev = mydate
end

MoveNearTh = function (p1,p2)
	local delta = math.rad(1)
	if (math.abs(p1-p2) < delta)
	 then return true
	 else return false
	end
end

MoveNow = function (obj,date)
 	local pos = obj.PosPrev + obj.V*(date-obj.TPrev)
	obj.TPrev = date
	obj.PosPrev = pos
	return pos
end

